#    Mode: Python tab-width: 4
#    Id: helper_misc.py
#    Author: David REGNIER
#    Description: Misc functions used by main class

# ======================================================================
# Copyright 2014 by David REGNIER
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name David
# REGNIER not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
# ======================================================================

import traceback
import py_compile
import os, sys, time, calendar
import xml.parsers.expat, uuid
from datetime import date
from datetime import datetime

"""Begin LogStats class"""
class LogStats(object):
    # Description: Constructor
    #              Init stats log file
    #              Call sample: sys.stdout = LogStats()
    # @return: void    
    def __init__(self, sOutputFile = os.getcwd() + "\\log\\" + 'xml_stats.log'):
        self.oConsole = sys.stdout
        self.sFile = open(sOutputFile, 'w')
    
    # Description: Write flow to console and file
    #
    # @param self: Constructor parameters
    # @param sFlow: String to write    
    # @return: int        
    def write(self, sFlow):
        try:
            if sFlow <> '':
                self.oConsole.write(sFlow) # Write to console
                self.sFile.write(sFlow) # Write to stats log file
            return 0
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1
"""End LogStats class"""

"""Begin ToolsBox class"""
class ToolsBox(object):
    # Description: Create XML node
    #              Call sample: ToolsBox.set_xml_node(document, "node_name", parent_node, ["node_content"], ["attribute_key"], ["attribute_content"])
    # @param oDoc: Current xml.dom.minidom.Document
    # @param sNodeName: XML node name
    # @param oParentNode: XML parent node
    # @param sNodeContent: (Optional) XML node content            
    # @param sXMLAttributeKey: (Optional) XML attribute key name
    # @param sXMLAttributeContent: (Optional) XML attribute content        
    # @return: mixed (object/int)
    @staticmethod 
    def set_xml_node(oDoc, sNodeName = '', oParentNode = '', sNodeContent = '', sXMLAttributeKey = '', sXMLAttributeContent = ''):
        try:
            oXMLNode = oDoc.createElement(sNodeName)
            if sXMLAttributeKey <> '':
                oXMLNode.setAttribute(sXMLAttributeKey, sXMLAttributeContent)
                oParentNode.appendChild(oXMLNode)
            if sNodeContent <> '':
                sXMLContent = oDoc.createTextNode(sNodeContent)
                oXMLNode.appendChild(sXMLContent)
            oParentNode.appendChild(oXMLNode)
            return oXMLNode
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1
            
    # Description: Return months list
    #              Call sample: ToolsBox.get_months()
    # @return: mixed (list/int)
    @staticmethod 
    def get_months():
        try:
            aMonths = ['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12']
            return aMonths
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1 
            
    # Description: Create folder tree for module
    #              Call sample: ToolsBox.init_folder_tree('file_xml', sFolder1, sFolder2, ...)
    # @param sSubFolder: Input folder    
    # @param *args: Folder1, Folder2, ...    
    # @return: int
    @staticmethod 
    def init_folder_tree(sSubFolder = '', *args):
        try:
            for element in args:
                if type(element) == str:
                    if not os.path.exists(element):
                        os.mkdir(element)
                # Deal with list element
                elif type(element) == list:
                    for item in element:
                        if not os.path.exists(sSubFolder + '/' + item):
                            os.mkdir(sSubFolder + '/' + item)
                                    
            return 0
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1    
    
    # Description: Build a UUID objects according to RFC 4122 based on the host ID and current time
    #              Call sample: ToolsBox.set_uuid()
    # @return: mixed (string/int)
    @staticmethod 
    def set_uuid():
        try:
            return str(uuid.uuid1())
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1
            
    # Description: Compile Python script
    #              Call sample: ToolsBox.init_compile(sInputPyScript = __file__)
    # @param sInputPyScript: Python Script file name    
    # @return: int
    @staticmethod 
    def init_compile(sInputPyScript = ''):
        try:
            if os.path.exists(sInputPyScript):
                py_compile.compile(sInputPyScript)                
            return 0
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1    
    
    # Description: Init error log file
    #              Call sample: ToolsBox.init_error_log()
    # @return: int
    @staticmethod 
    def init_error_log():
        try:
            if not os.path.exists('log'):
                os.makedirs('log')
            fileSocketError = open(os.getcwd() + "\\log\\" + 'xml_error.log', 'w')
            sys.stderr = fileSocketError
            return 0
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1
        
    # Description: XML validation
    #              Call sample: ToolsBox.check_xml_format(sOutputFile)
    # @param sOutputFile: XML output file
    # @return: boolean
    @staticmethod 
    def check_xml_format(sOutputFile = ''):
        try:
            if sOutputFile <> '':
                parser = xml.parsers.expat.ParserCreate()
                parser.ParseFile(open(sOutputFile, "r"))
            return True
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return False
        
    # Description: Check if file exist regarding his pattern name
    #              Call sample: ToolsBox.check_input_file(sInputFolder, sInputFile)
    # @param sInputFolder: Input folder
    # @param sInputFile: Input file    
    # @return: mixed (string/int)
    @staticmethod 
    def check_input_file(sInputFolder = '', sInputFile = ''):
        try:
            if sInputFolder <> '' and sInputFile <> '':
                i = 0
                for sFileName in os.listdir(sInputFolder):
                    if sInputFile[:5] in sFileName:
                        i += 1
                        sRealFileName = sFileName
                if i <> 1:
                    sys.stderr.write('ERROR: %s\n' % 'The input file seems to have a wrong name, or does not exist, or multiple existence')
                    sys.exit()
                else:
                    return sRealFileName
            else:
                sys.exit()
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1               
     
    # Description: Get first day of month
    #              Call sample: ToolsBox.get_first_day_of_month(date.today(), '-'))
    # @param oTodayDate: Current datetime object
    # @param sFormat: Current format, default /    
    # @return: mixed (string/int)
    @staticmethod 
    def get_first_day_of_month(oTodayDate, sFormat = '/'):
        try:
            return date(oTodayDate.year, oTodayDate.month, 1).strftime('%Y' + sFormat + '%m' + sFormat + '%d')    
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1
        
    # Description: Return current date, convert date string to datetime object
    #              Call sample: ToolsBox.get_current_date('02', '2012')
    # @param sMonthName: Month MM format    
    # @param sYearName: Year YYYY format    
    # @return: mixed (datetime/int)
    @staticmethod 
    def get_current_datetime_by_string(sMonthName, sYearName):
        try:
            sDate = sYearName + '-' + sMonthName + '-' + time.strftime('%d')
            return datetime.strptime(sDate, '%Y-%m-%d')
        except Exception:        
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1    
        
    # Description: Check for empty file
    #              Call sample: ToolsBox.check_empty_file(sInputFile)
    # @param sInputFile: Input file to check
    # @return: int
    @staticmethod 
    def check_empty_file(sInputFile = ''):
        try:
            if os.stat(sInputFile).st_size <> 0:
                return 0
            else:
                sys.stderr.write('ERROR: %s\n' % 'Input file is empty ' + sInputFile)
                sys.exit()                    
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1             

    # Description: Get end day of month
    #              Call sample: ToolsBox.get_end_day_of_month(date.today(), '-'))
    # @param oTodayDate: Current datetime object
    # @param sFormat: Current format, default /    
    # @return: mixed (string/int)
    @staticmethod 
    def get_end_day_of_month(oTodayDate, sFormat = '/'):
        try:
            iDaysNumberInMonth = calendar.monthrange(oTodayDate.year, oTodayDate.month)[1]
            return date(oTodayDate.year, oTodayDate.month, iDaysNumberInMonth).strftime('%Y' + sFormat + '%m' + sFormat + '%d')     
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1 
             
    # Description: Return current datetime
    #              Call sample: ToolsBox.get_current_datetime('%Y-%m-%dT%H:%M:%S')
    # @param sFormat: Format type    
    # @return: mixed (string/int)
    @staticmethod 
    def get_current_datetime(sFormat = ''):
        try:
            return time.strftime(sFormat)
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1
        
    # Description: Check for null element in input array
    #              Call sample: ToolsBox.check_empty_element_in_array(aInputArray, 6)
    # @param aInputArray: Input array
    # @param iNbElement: Number of element in array    
    # @return: int
    @staticmethod 
    def check_empty_element_in_array(aInputArray, iNbElement):
        try:
            if len(filter(bool, aInputArray)) == iNbElement:
                return 0
            else:
                sys.stderr.write('ERROR: %s\n' % 'One key element cannot be empty (you should have ' + str(iNbElement) + ' element(s)), in this array : ' + str(aInputArray))
                sys.exit()                    
        except Exception:
            sys.stderr.write('ERROR: %s\n' % traceback.format_exc())
            return 1        
"""End ToolsBox class"""